local SelectedIndex = 1;
local ClickedIndex = 1;
local SelectedID = nil;
local PlayerIndex = 1;

local professioncolors = {
	["NONE"] = "|cff808080",
	["ALT"] = "|cff8080ff",
	["GUILDON"] = "|cff40ff40",
	["GUILDOFF"] = "|cff408040",
	["FRIENDON"] = "|cffffffff",
}

local tradecolors = {
	["trivial"] = "|cff808080",
	["easy"] = "|cff00ff00",
	["medium"] = "|cffffff00",
	["optimal"] = "|cffff8000",
	["difficult"] = "|cffff0000",
};

local itemqualitystrings = {
	string.lower(ITEM_QUALITY0_DESC),
	string.lower(ITEM_QUALITY1_DESC),
	string.lower(ITEM_QUALITY2_DESC),
	string.lower(ITEM_QUALITY3_DESC),
	string.lower(ITEM_QUALITY4_DESC),
	string.lower(ITEM_QUALITY5_DESC),
	string.lower(ITEM_QUALITY6_DESC),
	string.lower(ITEM_QUALITY7_DESC),
};

local linktochattypes = {
	{"Link to:", nil, true},
	{"RAID", nil, false},
	{"PARTY", nil, false},
	{"GUILD", nil, false},
	{"SAY", nil, false},
	{"FRIEND", 1, false},
	{"WHISPERS", 1, false},
};

local Link_Data = {
	["Link to:"] = {},
	["RAID"] = {},
	["PARTY"] = {},
	["GUILD"] = {},
	["SAY"] = {},
	["FRIEND"] = {},
	["WHISPERS"] = {},
};

local Item_Type = {
	[BANKGUI_TYPE_CURRENCY] = "Currency",
	[BANKGUI_TYPE_FRIENDS] = "Friend",
	[BANKGUI_TYPE_GLYPHS] = "Glyph (equiped)",
	[BANKGUI_TYPE_FARMS] = "Farm Item",
	[BANKGUI_TYPE_TRADELINK] = "TradeLink",
	["-3"] = "Equiped",
	["-2"] = "Keyring",
	["-1"] = "Bankslot 1/28",
	["0"] = "Backpack",
	["1"] = "Bag 1",
	["2"] = "Bag 2",
	["3"] = "Bag 3",
	["4"] = "Bag 4",
	["5"] = "BankBag 1",
	["6"] = "BankBag 2",
	["7"] = "BankBag 3",
	["8"] = "BankBag 4",
	["9"] = "BankBag 5",
	["10"] = "BankBag 6",
	["11"] = "BankBag 7",
};

local Types_Data = {
	"Everything",
	"Items",
	"Tradeskills",
	"Profession Links",
	"Farm Reminders",
	"Glyphs",
	"Friends",
	"Currency",
};


-- Inserts all of the UI elements
function BankGUI:InsertUI()
	-- Add tab button to SocialFrame
	CreateFrame("Button", "FriendsFrameTab6", getglobal("FriendsDropDown"), "FriendsFrameTab6");
	PanelTemplates_SetNumTabs( getglobal("FriendsFrame"), (getglobal("FriendsFrame").numTabs + 1) );
	bank_numtab = getglobal("FriendsFrame").numTabs;

	if BankListOptions[GetRealmName()][1]["detach"] then
		-- Add the window in UIParent
		CreateFrame("Frame", "BankGUIFrame", getglobal("UIParent"), "BankGUIFrame");
		BankGUIFrame:SetFrameStrata("DIALOG");
	else
		-- Add the window in FriendsFrame
		CreateFrame("Frame", "BankGUIFrame", getglobal("FriendsFrame"), "BankGUIFrame");
		table.insert(FRIENDSFRAME_SUBFRAMES, "BankGUIFrame");
		BankGUIFrame:SetFrameStrata("HIGH");
	end

	FriendsFrameTab6:Enable();
	BankGUIDetachCheckButton_OnClick(false);
	BankGUIFrame:SetClampRectInsets(0, 0, 0, 0);
end


function BankGUI:SetSelectedBankGUI(index)
	SelectedIndex = index;
end


function BankGUI:GetSelectedBankGUI()
	return SelectedIndex;
end


function BankGUI:ShowTab()
	PanelTemplates_SetTab(FriendsFrame, bank_numtab);
	FriendsFrame_ShowSubFrame("BankGUIFrame");
	if BankListOptions[GetRealmName()][1]["detach"] then BankGUIFrame:Show() end
	BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
end


function BankGUI:ToggleTab()
	if BankListOptions[GetRealmName()][1]["detach"] then
		if (BankGUIFrame:IsVisible()) then BankGUIFrame:Hide() else BankGUIFrame:Show() end
	else
		ToggleFriendsFrame();
		if (FriendsFrame:IsVisible()) then
			getglobal("FriendsListFrame"):Hide();
			BankGUI:ShowTab();
		end
	end
end


function BankGUI:IsValidType(tempinfo)
	if tempinfo["type"] == nil then return false end
	if tonumber(tempinfo["type"]) == nil then return true end
	return ( tonumber(tempinfo["type"]) >= -3 )
end


function BankGUI:ResetFilter()
	CloseDropDownMenus();
	TypesIndex = 1;
	PlayerIndex = 1;
	SelectedID = nil;
	BankListOptions[GetRealmName()][1]["player_name"] = nil;
	BankListOptions[GetRealmName()][1]["player_type"] = 1;
	UIDropDownMenu_SetSelectedID(BankGUITypeDropDown, TypesIndex);
	UIDropDownMenu_SetText(BankGUITypeDropDown, Types_Data[TypesIndex]);
	UIDropDownMenu_SetSelectedID(BankGUIPlayerDropDown, PlayerIndex);
	UIDropDownMenu_SetText(BankGUIPlayerDropDown, "All Characters");
	BankGUIFilterTextBox:SetText("");
	BankGUIFilterTextBox:SetCursorPosition(0);
	UIDropDownMenu_EnableDropDown(BankGUIPlayerDropDown);
	BankGUIGuildCheckButton_OnClick(BankGUIGuildCheckButton);
end


function BankGUI:OnEnter()
if IsAltKeyDown() then return end
local bankinfo = BankGUI:GetItemByIndex(this:GetID());
local bankfaction = { 1,1,0.5 };
if not bankinfo then return end
if BankListOptions[GetRealmName()][1][BANKGUI_FACTION] then
	if bankinfo["type"] == BANKGUI_TYPE_FRIENDS then
		if bankinfo["faction"] == nil then bankinfo["faction"] = "Unknown" end
	end
	if bankinfo["faction"] ~= nil then
		bankfaction = BANKGUI_FACTION_COLORS[ bankinfo["faction"] ]
	elseif BankFactionColors[ bankinfo["name"] ] ~= nil then
		if BankFactionColors[ bankinfo["name"] ]["faction"] ~= nil then
			bankfaction = BANKGUI_FACTION_COLORS[ BankFactionColors[ bankinfo["name"] ]["faction"] ]
		else
			bankfaction = { BankFactionColors[ bankinfo["name"] ]["r"],BankFactionColors[ bankinfo["name"] ]["g"],BankFactionColors[ bankinfo["name"] ]["b"] }
		end
	else
		bankfaction = { 0.25,0.25,0.25 }
	end
end
GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT");
GameTooltip:ClearLines();
if bankinfo["type"] == BANKGUI_TYPE_GLYPHS or bankinfo["type"] == BANKGUI_TYPE_FRIENDS or bankinfo["type"] == BANKGUI_TYPE_CURRENCY then
	if IsShiftKeyDown() then
		if bankinfo["type"] == BANKGUI_TYPE_GLYPHS and bankinfo["item"] ~= nil then
			GameTooltip:SetHyperlink(bankinfo["item"]);
		end
	else
		if bankinfo["type"] ~= BANKGUI_TYPE_FRIENDS then GameTooltip:AddDoubleLine("Owner", bankinfo["name"], 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3]) end
		GameTooltip:AddDoubleLine(BANKGUI_TYPE, Item_Type[bankinfo["type"]], 1,1,0, 1,1,0.5);
		if bankinfo["type"] == BANKGUI_TYPE_FRIENDS then
			GameTooltip:AddDoubleLine("", bankinfo["faction"], 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3])
		end
		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("Shift-Click", "Put Itemlink in Chatframe", 1,1,0, 1,1,0.5);
		if bankinfo["type"] == BANKGUI_TYPE_GLYPHS then GameTooltip:AddDoubleLine("Shift-Mouseover","Display ingame Item tooltip", 1,1,0, 1,1,0.5) end
	end
elseif bankinfo["type"] == BANKGUI_TYPE_TRADELINK or bankinfo["type"] == BANKGUI_TYPE_FARMS then
	if bankinfo["type"] == BANKGUI_TYPE_FARMS then
		local itemName, itemLink = GetItemInfo(bankinfo["item"]);
		if itemLink then GameTooltip:SetHyperlink(itemLink)
		else GameTooltip:AddLine(bankinfo["item"])
		end
		GameTooltip:AddLine(" ");
	end
	GameTooltip:AddDoubleLine(BANKGUI_TYPE, Item_Type[bankinfo["type"]], 1,1,0, 1,1,0.5);
	GameTooltip:AddDoubleLine("Owner", bankinfo["name"], 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3])
	GameTooltip:AddLine(" ");
	GameTooltip:AddDoubleLine("Shift-Click", "Put Itemlink in Chatframe", 1,1,0, 1,1,0.5);
	if bankinfo["type"] == BANKGUI_TYPE_TRADELINK then
		professioncolors["GUILDON"] = bank_guildcolor_on;
		professioncolors["GUILDOFF"] = bank_guildcolor_of;
		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("No match", format("%sGrey|r",professioncolors["NONE"]), 1,1,0, 1,1,0.5);
		GameTooltip:AddDoubleLine("Alt Character", format("%sBlue|r",professioncolors["ALT"]), 1,1,0, 1,1,0.5);
		GameTooltip:AddDoubleLine("Online Guildmember", format("%sThis color|r",professioncolors["GUILDON"]), 1,1,0, 1,1,0.5);
		GameTooltip:AddDoubleLine("Offline Guildmember", format("%sThis color|r",professioncolors["GUILDOFF"]), 1,1,0, 1,1,0.5);
		GameTooltip:AddDoubleLine("Online Friend", format("%sWhite|r",professioncolors["FRIENDON"]), 1,1,0, 1,1,0.5);
	end
else
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(bankinfo["item"]);
	if itemStackCount == nil then itemStackCount = 1 end
	if itemTexture == nil then
		if string.find(bankinfo["item"],"Alchemy:") then itemSubType = "Alchemy"
		elseif string.find(bankinfo["item"],"Blacksmithing:") then itemSubType = "Blacksmithing"
		elseif string.find(bankinfo["item"],"Enchanting:") or string.find(bankinfo["item"],"Enchant ") then itemSubType = "Enchanting"
		elseif string.find(bankinfo["item"],"Engineering:") then itemSubType = "Engineering"
		elseif string.find(bankinfo["item"],"Jewelcrafting:") then itemSubType = "Jewelcrafting"
		elseif string.find(bankinfo["item"],"Leatherworking:") then itemSubType = "Leatherworking"
		elseif string.find(bankinfo["item"],"Tailoring:") then itemSubType = "Tailoring"
		elseif string.find(bankinfo["item"],"Mining:") then itemSubType = "Mining"
		elseif string.find(bankinfo["item"],"Cooking:") then itemSubType = "Cooking"
		elseif string.find(bankinfo["item"],"First Aid:") then itemSubType="First Aid"
		end
	end
	if itemSubType == nil then itemSubType = "" end
	if itemType == nil then
		itemType = itemSubType;
		itemSubType = "";
	end
	if IsShiftKeyDown() then
		GameTooltip:SetHyperlink(bankinfo["item"]);
		GameTooltip:AddLine(" ");
	else
		GameTooltip:AddLine(bankinfo["item"]);
		if not BankListOptions[GetRealmName()][1]["guild_checked"] then
			GameTooltip:AddDoubleLine("Owner", bankinfo["name"], 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3]);
		end
		if itemStackCount ~= 1 then GameTooltip:AddDoubleLine(BANKGUI_AMOUNT, format("%i / %i", bankinfo["amount"], itemStackCount), 1,1,0, 1,1,0.5) end
		if TypesIndex ~= 4 and bankinfo["bank"] == true then
			GameTooltip:AddDoubleLine(BANKGUI_TYPE, format("%s,%s",itemType,itemSubType), 1,1,0, 1,1,0.5);
			local bank_level = "";
			if itemLevel and itemMinLevel then
				bank_level = bank_level ..format("%i / %i", itemMinLevel, itemLevel);
			elseif itemLevel then
				bank_level = bank_level ..itemLevel;
			elseif itemMinLevel then
				bank_level = bank_level ..itemMinLevel;
			end
			if itemRarity then
				if bank_level ~= "" then bank_level = bank_level ..", " end
				bank_level = bank_level ..itemqualitystrings[itemRarity+1];
			end
			GameTooltip:AddDoubleLine(BANKGUI_LEVEL, bank_level, 1,1,0, 1,1,0.5);
			if BankListOptions[GetRealmName()][1]["guild_checked"] then
				if bankinfo["type"] ~= nil then GameTooltip:AddDoubleLine(BANKGUI_INVENT, "Guildbank Tab " ..bankinfo["type"], 1,1,0,1,1,0.5) end
			else
				if not BankListOptions[GetRealmName()][1][BANKGUI_TOTALS] or itemStackCount == 1 then
					if bankinfo["type"] ~= nil then GameTooltip:AddDoubleLine(BANKGUI_INVENT, Item_Type[bankinfo["type"]], 1,1,0,1,1,0.5) end
				else
					GameTooltip:AddDoubleLine(BANKGUI_INVENT, "-", 1,1,0,1,1,0.5)
				end
			end
		else
			GameTooltip:AddDoubleLine(BANKGUI_TYPE, format("%s,%s", itemType, itemSubType), 1,1,0, 1,1,0.5);
			if bankinfo["type"] ~= nil then
				if tradecolors[bankinfo["type"]] then GameTooltip:AddDoubleLine(BANKGUI_LEVEL, format("%s%s|r", tradecolors[bankinfo["type"]], bankinfo["type"]), 1,1,0, 1,1,0.5)
				else GameTooltip:AddDoubleLine(BANKGUI_LEVEL, bankinfo["type"], 1,1,0, 1,1,0.5);
				end
				GameTooltip:AddDoubleLine(BANKGUI_INVENT, "Tradeskill", 1,1,0, 1,1,0.5);
			end
		end
		if (itemEquipLoc and _G[itemEquipLoc]) then
			GameTooltip:AddDoubleLine(BANKGUI_EQUIP, _G[itemEquipLoc], 1,1,0, 1,1,0.5);
		end
		if itemStackCount ~= 1 and BankListOptions[GetRealmName()][1][BANKGUI_TOOLTIPS] then
			BankGUI:AddTooltip(GameTooltip, itemName);
		end
		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("Shift-Mouseover","Display ingame Item tooltip", 1,1,0, 1,1,0.5);
	end
	GameTooltip:AddDoubleLine("Shift-Click", "Put Itemlink in Chatframe", 1,1,0, 1,1,0.5);
	GameTooltip:AddDoubleLine("Control-Click", "Dressup Item", 1,1,0,1,1,0.5);
	GameTooltip:AddDoubleLine("Alt-Click", "Put Itemlink in Farm Item name", 1,1,0, 1,1,0.5);
	GameTooltip:AddDoubleLine("Right-Click", "Link via menu", 1,1,0, 1,1,0.5);
end
GameTooltip:Show();
end


function BankGUI:OnClickBankGUIButton(button)
	local index = this:GetID();
	local bankinfo = BankGUI:GetItemByIndex(index);
	HideDropDownMenu(1);
	if IsControlKeyDown() and IsAltKeyDown() and IsEquippableItem(bankinfo["item"]) then
		if DemoFrame then model:TryOn(bankinfo["item"]) end
	elseif IsControlKeyDown() and IsEquippableItem(bankinfo["item"]) then
		DressUpItemLink(bankinfo["item"]);
	elseif IsAltKeyDown() then
		if not BankGUIFrame:IsVisible() then BankGUI:ToggleTab() end
		BankGUIFarmFrame:Show();
		BankGUIFarmItemBox:SetText("");
		BankGUIFarmItemBox:Insert(bankinfo["item"]);
		BankGUIFarmItemBox:SetCursorPosition(0);
		if BankGUIFarmAmountBox:GetNumber() == 0 then BankGUIFarmAmountBox:Insert("1") end
		BankGUIFarmAmountBox:SetFocus();
	elseif button == "LeftButton" then
		BankGUI:SetSelectedBankGUI(index);
		BankGUI:UpdateUI();
		if IsShiftKeyDown() then
			if ChatFrameEditBox:IsVisible() then ChatFrameEditBox:Insert(bankinfo["item"])
			elseif bankinfo["type"] == BANKGUI_TYPE_TRADELINK and not BankListOptions[GetRealmName()][1]["guild_checked"] then
				bank_chat:AddMessage( format("%s %s |cffffff80(%s)|r", bankinfo["name"], bankinfo["item"], bankinfo["amount"]), 0.5,1,1 )
			else
				bank_chat:AddMessage(bankinfo["item"])
			end
		elseif BankGUI:IsValidType(bankinfo) then
			BankGUI:ShowDetails();
		else
			BankGUIDetailsFrame:Hide();
		end
	elseif button == "RightButton" then
		if bankinfo["type"] ~= BANKGUI_TYPE_FARMS then
			ClickedIndex = index;
			BankGUILinkDropDown.initialize = BankGUILinkDropDown_Initialize;
			BankGUILinkDropDown.displayMode = "MENU";
			ToggleDropDownMenu(1, nil, BankGUILinkDropDown, "cursor");
		end
	end
end


function BankGUI:ShowDetails()
	-- get info
	local bankinfo = BankGUI:GetItemByIndex(BankGUI:GetSelectedBankGUI());
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(bankinfo["item"]);
	local itemtx = getglobal("BankGUIDetailsItemText");
	local textur = getglobal("BankGUIDetailsIconTexture");
	local amount = getglobal("BankGUIDetailsAmountText");
	local ilevel = getglobal("BankGUIDetailsLevelText");
	local ilocat = getglobal("BankGUIDetailsLocaText");
	local iequip = getglobal("BankGUIDetailsEquiText");
	local prefix = "Interface\\Icons\\";

	local bankfaction = { 1.0,1.0,1.0 };
	if BankListOptions[GetRealmName()][1][BANKGUI_FACTION] then
		if bankinfo["faction"] ~= nil then
			bankfaction = BANKGUI_FACTION_COLORS[ bankinfo["faction"] ]
		elseif BankFactionColors[ bankinfo["name"] ] ~= nil then
			if BankFactionColors[ bankinfo["name"] ]["faction"] ~= nil then
				bankfaction = BANKGUI_FACTION_COLORS[ BankFactionColors[ bankinfo["name"] ]["faction"] ]
			else
				bankfaction = { BankFactionColors[ bankinfo["name"] ]["r"],BankFactionColors[ bankinfo["name"] ]["g"],BankFactionColors[ bankinfo["name"] ]["b"] }
			end
		else
			bankfaction = { 0.25,0.25,0.25 }
		end
	end

	-- update details
	if itemLink then itemtx:SetText( itemLink )
	else itemtx:SetText( bankinfo["item"] ) end
	if itemStackCount == nil then itemStackCount = 1 end
	if itemTexture == nil then itemTexture = GetItemIcon(bankinfo["item"]) end
	if itemTexture == nil then
		if string.find(bankinfo["item"],"Alchemy:") then itemTexture = prefix.."Trade_Alchemy";itemSubType = "Alchemy";
		elseif string.find(bankinfo["item"],"Blacksmithing:") then itemTexture = prefix.."Trade_Blacksmithing";itemSubType = "Blacksmithing";
		elseif string.find(bankinfo["item"],"Enchanting:") or string.find(bankinfo["item"],"Enchant ") then itemTexture = prefix.."Trade_Engraving";itemSubType = "Enchanting";
		elseif string.find(bankinfo["item"],"Engineering:") then itemTexture = prefix.."Trade_Engineering";itemSubType = "Engineering";
		elseif string.find(bankinfo["item"],"Inscription:") then itemTexture = prefix.."INV_Inscription_Tradeskill01";itemSubType = "Inscription";
		elseif string.find(bankinfo["item"],"Jewelcrafting:") then itemTexture = prefix.."INV_Misc_Gem_02";itemSubType = "Jewelcrafting";
		elseif string.find(bankinfo["item"],"Leatherworking:") then itemTexture = prefix.."Trade_Leatherworking";itemSubType = "Leatherworking";
		elseif string.find(bankinfo["item"],"Tailoring:") then itemTexture = prefix.."Trade_Tailoring";itemSubType = "Tailoring";
		elseif string.find(bankinfo["item"],"Mining:") then itemTexture = prefix.."Trade_Mining";itemSubType = "Mining";
		elseif string.find(bankinfo["item"],"Cooking:") then itemTexture = prefix.."INV_Misc_Food_15";itemSubType = "Cooking";
		elseif string.find(bankinfo["item"],"First Aid:") then itemTexture = prefix.."Spell_Holy_SealOfSacrifice";itemSubType = "First Aid";
		end
	end
	textur:SetTexture(itemTexture);
	if itemSubType == nil then itemSubType = "" end
	if itemType == nil then
		itemType = itemSubType;
		itemSubType = "";
	end
	textur:SetTexCoord(0, 1, 0, 1);
	if bankinfo["bank"] == true then
		if itemStackCount ~= 1 then amount:SetText( format("%i / %i", bankinfo["amount"], itemStackCount) )
		else amount:SetText( format("%i", bankinfo["amount"]) )
		end
		if itemMinLevel and itemRarity then ilevel:SetText( format("%i, %s", itemMinLevel, itemqualitystrings[itemRarity+1]) )
		elseif itemMinLevel then ilevel:SetText( "" ..itemMinLevel )
		else ilevel:SetText( "" )
		end
	else
		amount:SetText( "" );
		if bankinfo["type"] ~= nil then
			if tradecolors[bankinfo["type"]] then ilevel:SetText( format("%s%s|r",tradecolors[bankinfo["type"]],bankinfo["type"]) )
			elseif bankinfo["type"] == BANKGUI_TYPE_GLYPHS then ilevel:SetText( "-" )
			else ilevel:SetText( bankinfo["type"] )
			end
		else ilevel:SetText( "" )
		end
	end
	getglobal("BankGUIDetailsTypeText"):SetText( itemType ..", " ..itemSubType );
	if BankListOptions[GetRealmName()][1]["guild_checked"] then
		if bankinfo["type"] ~= nil then ilocat:SetText( "Guildbank Tab " ..bankinfo["type"] )
		else ilocat:SetText( "-" )
		end
		if (itemEquipLoc and _G[itemEquipLoc]) then iequip:SetText( _G[itemEquipLoc] )
		else iequip:SetText( "-" )
		end
	else
		getglobal("BankGUIDetailsNameText"):SetTextColor(bankfaction[1],bankfaction[2],bankfaction[3]);
		getglobal("BankGUIDetailsNameText"):SetText( bankinfo["name"] );
		if bankinfo["bank"] == true then
			if not BankListOptions[GetRealmName()][1][BANKGUI_TOTALS] or itemStackCount == 1 then
				ilocat:SetText( Item_Type[bankinfo["type"]] );
			else
				ilocat:SetText( "-" );
			end
		elseif bankinfo["type"] == BANKGUI_TYPE_GLYPHS then ilocat:SetText( "Glyph socket" )
		else ilocat:SetText( format("|cffffffff%s|r", "Tradeskill") )
		end
		if (itemEquipLoc and _G[itemEquipLoc]) then iequip:SetText( _G[itemEquipLoc] )
		elseif bankinfo["type"] == BANKGUI_TYPE_GLYPHS then iequip:SetText( bankinfo["bank"] )
		else iequip:SetText( "-" )
		end
	end
	BankGUIDetailsFrame:Show();
end


function BankGUIDetachCheckButton_OnClick(bank_showframe)
	CloseDropDownMenus();
	BankListOptions[GetRealmName()][1]["detach"] = BankGUIDetachCheckButton:GetChecked();
	if BankListOptions[GetRealmName()][1]["detach"] then
		if FRIENDSFRAME_SUBFRAMES[table.getn(FRIENDSFRAME_SUBFRAMES)] == "BankGUIFrame" then table.remove(FRIENDSFRAME_SUBFRAMES) end
		BankGUIFrame:SetParent("UIParent");
		BankGUIFrame:SetFrameStrata("DIALOG");
		BankGUICloseButton:Show();
		BankGUIFrame:SetMovable(true);
		BankGUIFrame:EnableMouse(true);
		BankGUIFrame:RegisterForDrag("LeftButton");
		BankGUIFrame:ClearAllPoints();
		if BankListOptions[GetRealmName()][1]["pos"] then
			BankGUIFrame:SetPoint(BankListOptions[GetRealmName()][1]["pos"]["point"], "UIParent", BankListOptions[GetRealmName()][1]["pos"]["relativePoint"], BankListOptions[GetRealmName()][1]["pos"]["xOfs"], BankListOptions[GetRealmName()][1]["pos"]["yOfs"])
		else
			BankGUIFrame:SetPoint("TOPLEFT", "UIParent", "CENTER", -(BankGUIFrame:GetWidth()/2), (BankGUIFrame:GetHeight()/2))
		end
		if bank_showframe == nil then BankGUIFrame:Show() end
	else
		if FRIENDSFRAME_SUBFRAMES[table.getn(FRIENDSFRAME_SUBFRAMES)] ~= "BankGUIFrame" then table.insert(FRIENDSFRAME_SUBFRAMES, "BankGUIFrame") end
		BankGUIFrame:SetParent("FriendsFrame");
		BankGUIFrame:SetFrameStrata("HIGH");
		BankGUICloseButton:Hide();
		BankGUIFrame:SetMovable(false);
		BankGUIFrame:EnableMouse(false);
		BankGUIFrame:RegisterForDrag();
		BankGUIFrame:ClearAllPoints();
		BankGUIFrame:SetAllPoints();
	end
	BankGUI:SetBankFrameBackGround();
end


function BankGUIOptionsButton_OnEnter()
	BankGUIDetailsFrame:Hide();
	BankGUIFarmFrame:Hide();
	BankGUIOptionsDropDown.initialize = BankGUIOptionsDropDown_Initialize;
	BankGUIOptionsDropDown.displayMode = "MENU";
	ToggleDropDownMenu(1, nil, BankGUIOptionsDropDown, "BankGUIOptionsButton", 0, 0);
end


function BankGUIOptionsDropDown_Initialize(self, level)
	local optionsmenuinfo = UIDropDownMenu_CreateInfo();
	local i, subarray, subsubarray;
	if level == nil or level == 1 then
		i = 1;
		for subarray in pairs(BANKGUI_OPTIONS_MENU) do
			if subarray == BANKGUI_KEEPTRACKOF or string.find(subarray, "Sort Order") ~= nil then
				if i == 2 then subarray = BANKGUI_SORTTITLE1 end
				if i == 3 then subarray = BANKGUI_SORTTITLE2 end
				if i == 4 then subarray = BANKGUI_SORTTITLE3 end
				optionsmenuinfo.text = subarray;
				optionsmenuinfo.value = { ["Level1_Key"] = subarray };
				optionsmenuinfo.func = nil;
				optionsmenuinfo.owner = this:GetParent();
				optionsmenuinfo.checked = nil;
				optionsmenuinfo.hasArrow = true;
				optionsmenuinfo.nested = 1;
				optionsmenuinfo.notCheckable = true;
				optionsmenuinfo.isTitle = false;
				UIDropDownMenu_AddButton(optionsmenuinfo, level);
				i = i + 1;
			end
		end
		for i = 1, table.getn(BANKGUI_OPTIONS_MENU) do
			optionsmenuinfo.text = BANKGUI_OPTIONS_MENU[i]["name"];
			optionsmenuinfo.value = i;
			optionsmenuinfo.func = BankGUIOptionsDropDown_Select;
			optionsmenuinfo.owner = this:GetParent();
			if BANKGUI_OPTIONS_MENU[i]["checkable"] then
				optionsmenuinfo.checked = BankListOptions[GetRealmName()][1][ BANKGUI_OPTIONS_MENU[i]["name"] ];
				optionsmenuinfo.notCheckable = false;
			else
				optionsmenuinfo.checked = false;
				optionsmenuinfo.notCheckable = true;
			end
			optionsmenuinfo.hasArrow = false;
			optionsmenuinfo.nested = 0;
			optionsmenuinfo.isTitle = false;
			UIDropDownMenu_AddButton(optionsmenuinfo, level);
		end
	elseif level == 2 then
		subarray = UIDROPDOWNMENU_MENU_VALUE["Level1_Key"];
		for subsubarray in pairs(BANKGUI_OPTIONS_MENU[subarray]) do
			optionsmenuinfo.text = subsubarray;
			optionsmenuinfo.value = { ["Level1_Key"] = subarray; ["Sublevel_Key"] = subsubarray; };
			optionsmenuinfo.func = BankGUIOptionsDropDown_Select;
			optionsmenuinfo.owner = this:GetParent();
			if subarray == BANKGUI_KEEPTRACKOF then
				optionsmenuinfo.checked = BankListOptions[GetRealmName()][1][BANKGUI_KEEPTRACKOF][ subsubarray ];
			elseif string.find(subarray, "Sort Order") ~= nil then
				if subsubarray == BankListOptions[GetRealmName()][1][subarray]["name"] then
					optionsmenuinfo.checked = true
				else
					optionsmenuinfo.checked = false
				end
			end
			optionsmenuinfo.hasArrow = false;
			optionsmenuinfo.notCheckable = false;
			optionsmenuinfo.isTitle = false;
			optionsmenuinfo.disabled = nil;
			UIDropDownMenu_AddButton(optionsmenuinfo, level);
		end
	end
end


local function BankGUI_ShowColorPicker(r, g, b, a, myColorPickerCallback)
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = myColorPickerCallback, myColorPickerCallback, myColorPickerCallback;
	ColorPickerFrame:SetColorRGB(r, g, b);
	ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = ( a ~= nil ), a;
	ColorPickerFrame.previousValues = {r, g, b, a};
	ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
	ColorPickerFrame:Show();
end


local function BankGUI_BarColorCallback(restore)
	local newR, newG, newB, newA;
	if restore then
		newR, newG, newB, newA = unpack(restore);
	else
		newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
	end
	BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["r"], BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["g"], BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["b"] = newR, newG, newB;
	if newA ~= nil then BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["a"] = newA end
	BankGUI:SetBankGUIBarColor();
end


local function BankGUI_BackColorCallback(restore)
	local newR, newG, newB, newA;
	if restore then
		newR, newG, newB, newA = unpack(restore);
	else
		newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
	end
	BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["r"], BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["g"], BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["b"], BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["a"] = newR, newG, newB, newA;
	if newA ~= nil then BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["a"] = newA end
	BankGUI:SetBankGUIBackgroundColor();
end


local bank_option_value = "";

function BankGUIOptionsDropDown_Select()
	CloseDropDownMenus();
	if tonumber(this.value) ~= nil and BANKGUI_OPTIONS_MENU[this.value] and BANKGUI_OPTIONS_MENU[this.value]["name"] ~= nil then
		if BANKGUI_OPTIONS_MENU[this.value]["name"] == BANKGUI_BARCOLOR then
			BankGUI_ShowColorPicker(BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["r"], BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["g"], BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["b"], BankListOptions[GetRealmName()][1][ BANKGUI_BARCOLOR ]["a"], BankGUI_BarColorCallback);
		elseif BANKGUI_OPTIONS_MENU[this.value]["name"] == BANKGUI_BACKCOLOR then
			BankGUI_ShowColorPicker(BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["r"], BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["g"], BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["b"], BankListOptions[GetRealmName()][1][ BANKGUI_BACKCOLOR ]["a"], BankGUI_BackColorCallback);
		elseif BANKGUI_OPTIONS_MENU[this.value]["name"] == BANKGUI_RESETCOLORS then
			BankGUI:ResetBankGUIBarColor();
			BankGUI:ResetBankGUIBackgroundColor();
			BankGUI:SetBankGUIBarColor();
			BankGUI:SetBankGUIBackgroundColor();
		else
			BankListOptions[GetRealmName()][1][ BANKGUI_OPTIONS_MENU[this.value]["name"] ] = not BankListOptions[GetRealmName()][1][ BANKGUI_OPTIONS_MENU[this.value]["name"] ];
			if BANKGUI_OPTIONS_MENU[this.value]["name"] == BANKGUI_FACTION then
				PlayerIndex = 1;
				BankListOptions[GetRealmName()][1]["player_name"] = nil;
				UIDropDownMenu_SetSelectedID(BankGUIPlayerDropDown, PlayerIndex);
				UIDropDownMenu_SetText(BankGUIPlayerDropDown, "All Characters");
				BankGUIGuildCheckButton_OnClick(BankGUIGuildCheckButton);
				BankGUI:UpdateStock();
				BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
			end
		end
		BankGUI:SetBankFrameBackGround();
	elseif UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ~= nil then
		local i, subarray;
		i = 1;
		for subarray in pairs(BANKGUI_OPTIONS_MENU[ UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ]) do
			if i == this:GetID() then bank_option_value = subarray end
			i = i + 1;
		end
		if UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] == BANKGUI_KEEPTRACKOF then
			BankListOptions[GetRealmName()][1][BANKGUI_KEEPTRACKOF][bank_option_value] = not BankListOptions[GetRealmName()][1][BANKGUI_KEEPTRACKOF][bank_option_value];
			if BankListOptions[GetRealmName()][1][BANKGUI_KEEPTRACKOF][bank_option_value] == true then return end
			StaticPopupDialogs["DELETE_ALL"] = {
				text = "|cffffff00Do you want to delete all saved info of type |r%s|cffffff00 ?\n\n( NOTE: This is for all Characters on this account. )|r",
				button1 = "Yes",
				button2 = "No",
				OnAccept = function()
					BankGUIOptionsButton_OnYes();
				end,
				timeout = 0,
				whileDead = 1,
				showAlert = 1,
				hideOnEscape = 1
			};
			StaticPopup_Show("DELETE_ALL", bank_option_value);
		elseif UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ~= nil and string.find(UIDROPDOWNMENU_MENU_VALUE["Level1_Key"], "Sort Order") ~= nil then
			BankListOptions[GetRealmName()][1][ UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ]["name"] = bank_option_value;
		end
		BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
	end
end


function BankGUIOptionsButton_OnYes()
	local i, bankinfo, bank_keepremoving = 1, nil, true;
	if bank_option_value == BANKGUI_KEEPGUILD then
		BankListGuildItems[GetRealmName()] = {};
	elseif bank_option_value == BANKGUI_KEEPITEMS then
		if table.getn(BankListItems[GetRealmName()]) ~= 0 then
			while bank_keepremoving do
				bankinfo = BankListItems[GetRealmName()][i];
				if bankinfo["bank"] and BankGUI:IsValidType(bankinfo) then
					table.remove(BankListItems[GetRealmName()], i);
					i = 1;
				else
					i = i + 1;
				end	
				if i > table.getn(BankListItems[GetRealmName()]) then bank_keepremoving = false end
			end
		end
	elseif bank_option_value == BANKGUI_KEEPTRADE then
		if table.getn(BankListItems[GetRealmName()]) ~= 0 then
			while bank_keepremoving do
				bankinfo = BankListItems[GetRealmName()][i];
				if string.find(bankinfo["item"],"|Henchant") ~= nil then
					table.remove(BankListItems[GetRealmName()], i);
					i = 1;
				else
					i = i + 1;
				end
				if i > table.getn(BankListItems[GetRealmName()]) then bank_keepremoving = false end
			end
		end
	elseif bank_option_value == BANKGUI_KEEPPROFS then
		if table.getn(BankListItems[GetRealmName()]) ~= 0 then
			while bank_keepremoving do
				bankinfo = BankListItems[GetRealmName()][i];
				if bankinfo["type"] == BANKGUI_TYPE_TRADELINK then
					table.remove(BankListItems[GetRealmName()], i);
					i = 1;
				else
					i = i + 1;
				end
				if i > table.getn(BankListItems[GetRealmName()]) then bank_keepremoving = false end
			end
		end
	elseif bank_option_value == BANKGUI_KEEPGLYPS then
		BankListGlyphs[GetRealmName()] = {};
	elseif bank_option_value == BANKGUI_KEEPFRIEND then
		BankListFriends[GetRealmName()] = {};
	elseif bank_option_value == BANKGUI_KEEPCURR then
		BankListCurrency[GetRealmName()] = {};
	end
	BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
end


function BankGUIGuildCheckButton_OnClick(this)
	local i;
	CloseDropDownMenus();
	BankListOptions[GetRealmName()][1]["guild_checked"] = this:GetChecked();
	PlayerIndex = 1;
	if BankListOptions[GetRealmName()][1]["guild_checked"] then
		TypesIndex = 1;
		SelectedID = BankListOptions[GetRealmName()][1]["guild_name"];
		BankGUIPlayerTitle:SetText("Guild:");
		UIDropDownMenu_DisableDropDown(BankGUITypeDropDown);
		for i = 1, BankGUI:GetNumGuilds() do
			if BankListOptions[GetRealmName()][1]["guild_name"] and BankGUI:GetGuild(i) == BankListOptions[GetRealmName()][1]["guild_name"] then PlayerIndex = i end
		end
	else
		if BankListOptions[GetRealmName()][1]["player_name"] then SelectedID = BankListOptions[GetRealmName()][1]["player_name"]
		else SelectedID = nil end
		if BankListOptions[GetRealmName()][1]["player_type"] then TypesIndex = BankListOptions[GetRealmName()][1]["player_type"] end
		BankGUIPlayerTitle:SetText("Char");
		UIDropDownMenu_EnableDropDown(BankGUITypeDropDown);
		for i = 1, table.getn(BankListPlayers[GetRealmName()]) do
			if SelectedID and BankListPlayers[GetRealmName()][i]["name"] == SelectedID then PlayerIndex = (i + 1) end
		end
	end
	UIDropDownMenu_SetSelectedID(BankGUITypeDropDown, TypesIndex);
	UIDropDownMenu_Initialize(BankGUIPlayerDropDown, BankGUIPlayerDropDown_Initialize);
	UIDropDownMenu_SetSelectedID(BankGUIPlayerDropDown, PlayerIndex);
	if BankListOptions[GetRealmName()][1]["guild_checked"] then
		SelectedID = BankGUI:GetGuild(PlayerIndex);
		UIDropDownMenu_SetText(BankGUITypeDropDown, Types_Data[1]);
		UIDropDownMenu_SetText(BankGUIPlayerDropDown, SelectedID);
		if SelectedID == "" then BankGUIDeleteButton:Disable()
		else BankGUIDeleteButton:Enable() end
		BankGUIDeleteButton:SetText(BANKGUI_DELETES);
	else
		UIDropDownMenu_SetText(BankGUITypeDropDown, Types_Data[TypesIndex]);
		if PlayerIndex == 1 then
			UIDropDownMenu_SetText(BankGUIPlayerDropDown, "All Characters");
			SelectedID = nil;
			BankListOptions[GetRealmName()][1]["player_name"] = nil;
			if TypesIndex == 4 or TypesIndex == 5 or TypesIndex == 7 then BankGUIDeleteButton:Enable()
			else BankGUIDeleteButton:Disable() end
		else
			SelectedID = BankListPlayers[GetRealmName()][PlayerIndex-1]["name"];
			UIDropDownMenu_SetText(BankGUIPlayerDropDown, SelectedID);
			BankGUIDeleteButton:Enable();
		end
		if TypesIndex == 6 or TypesIndex == 8 then
			BankGUIDeleteButton:Disable();
		end
		if TypesIndex == 4 or TypesIndex == 5 or TypesIndex == 7 then BankGUIDeleteButton:SetText(BANKGUI_DELETE)
		else BankGUIDeleteButton:SetText(BANKGUI_DELETES)
		end
		if TypesIndex == 7 then
			UIDropDownMenu_DisableDropDown(BankGUIPlayerDropDown);
		else
			UIDropDownMenu_EnableDropDown(BankGUIPlayerDropDown);
		end
	end
	BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
end


function BankGUIDeleteButton_OnClick()
	CloseDropDownMenus();
	if table.getn(BankFilter) == 0 then return end
	StaticPopupDialogs["DELETE_CHARACTER"] = {
		text = "|cffffff00Are you sure you want to delete all items of|r %s|cffffff00 from the list ?\n\n( NOTE:|r %s|cffffff00 will also be removed from the dropdown menu's )|r",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			BankGUIDeleteButton_OnYes();
		end,
		timeout = 0,
		whileDead = 1,
		which = "",
		showAlert = 1,
		hideOnEscape = 1
	};
	if TypesIndex == 4 or TypesIndex == 5 then
		StaticPopupDialogs["DELETE_CHARACTER"].text = "|cffffff00Are you sure you want to delete|r %s's %s|cffffff00 from the list ?|r";
		StaticPopup_Show("DELETE_CHARACTER", BankFilter[SelectedIndex]["name"], BankFilter[SelectedIndex]["item"]);
	elseif TypesIndex == 7 then
		StaticPopupDialogs["DELETE_CHARACTER"].text = "|cffffff00Are you sure you want to delete|r %s|cffffff00 from the list ?|r";
		StaticPopup_Show("DELETE_CHARACTER", BankFilter[SelectedIndex]["name"]);
	else StaticPopup_Show("DELETE_CHARACTER", SelectedID, SelectedID) end
end


function BankGUIDeleteButton_OnYes()
  local i, itemname, bank_itemindexbyname;
  CloseDropDownMenus();
  BankGUIDetailsFrame:Hide();
  if BankListOptions[GetRealmName()][1]["guild_checked"] then
	for i = 1, BankGUI:GetNumGuilds() do
		if BankListGuildInfo[GetRealmName()][i] and BankListGuildInfo[GetRealmName()][i]["name"] == SelectedID then table.remove(BankListGuildInfo[GetRealmName()], i) end
	end
	if not BankListGuildItems[GetRealmName()] then return end
	if not BankListGuildItems[GetRealmName()][SelectedID] then return end
	while BankListGuildItems[GetRealmName()][SelectedID][1] do
		table.remove( BankListGuildItems[GetRealmName()][SelectedID] );
	end
	if BankListOptions[GetRealmName()][1]["guild_name"] and BankListOptions[GetRealmName()][1]["guild_name"] == SelectedID then
		BankListOptions[GetRealmName()][1]["guild_name"] = BankGUI:GetGuild(1);
		SelectedID = BankListOptions[GetRealmName()][1]["guild_name"];
	end
  else
	if TypesIndex == 4 then
		for i = 1, table.getn(BankListItems[GetRealmName()]) do
			if BankListItems[GetRealmName()][i] and BankListItems[GetRealmName()][i]["type"] == BANKGUI_TYPE_TRADELINK and BankListItems[GetRealmName()][i]["name"] == BankFilter[SelectedIndex]["name"] and BankListItems[GetRealmName()][i]["item"] == BankFilter[SelectedIndex]["item"] then table.remove(BankListItems[GetRealmName()], i) end
		end
	elseif TypesIndex == 5 then
		for i = 1, table.getn(BankListFarms[GetRealmName()]) do
			if BankListFarms[GetRealmName()][i] and BankListFarms[GetRealmName()][i]["name"] == BankFilter[SelectedIndex]["name"] and BankListFarms[GetRealmName()][i]["item"] == BankFilter[SelectedIndex]["item"] and BankListFarms[GetRealmName()][i]["amount"] == BankFilter[SelectedIndex]["amount"] then
				itemname = GetItemInfo(BankFilter[SelectedIndex]["item"]);
				if BankListStock[itemname] and BankListStock[itemname]["farm"] then
					BankListStock[itemname]["farm"]["amount"] = BankListStock[itemname]["farm"]["amount"] - BankListFarms[GetRealmName()][i]["amount"];
					if BankListStock[itemname]["farm"]["amount"] <= 0 then BankListStock[itemname]["farm"] = nil end
				end
				table.remove(BankListFarms[GetRealmName()], i);
				break;
			end
		end
	elseif TypesIndex == 7 then
		for i = 1, table.getn(BankListFriends[GetRealmName()]) do
			if BankListFriends[GetRealmName()][i] and BankListFriends[GetRealmName()][i]["name"] == BankFilter[SelectedIndex]["name"] then
				table.remove(BankListFriends[GetRealmName()], i);
				break;
			end
		end
	else
		while true do
			bank_itemindexbyname = BankGUI:GetBankItemIndexByName(SelectedID);
			if bank_itemindexbyname == 0 then break end
			table.remove( BankListItems[GetRealmName()], bank_itemindexbyname );
		end
		for i = 1, table.getn(BankListPlayers[GetRealmName()]) do
			if BankListPlayers[GetRealmName()][i] and BankListPlayers[GetRealmName()][i]["name"] == SelectedID then table.remove(BankListPlayers[GetRealmName()], i) end
		end
	end
  end
  BankGUIGuildCheckButton_OnClick(BankGUIGuildCheckButton);
  BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
end


function BankGUITypeDropDown_Initialize()
	local i;
	local typemenuinfo = UIDropDownMenu_CreateInfo();
	for i = 1, table.getn(Types_Data) do
		typemenuinfo.text = Types_Data[i];
		typemenuinfo.value = Types_Data[i];
		typemenuinfo.func = BankGUITypeDropDown_Select;
		typemenuinfo.owner = this:GetParent();
		typemenuinfo.checked = nil;
		typemenuinfo.hasArrow = false;
		typemenuinfo.notCheckable = false;
		UIDropDownMenu_AddButton(typemenuinfo);
 	end
end


function BankGUITypeDropDown_Select()
	UIDropDownMenu_SetSelectedID(this.owner, this:GetID());
	TypesIndex = this:GetID();
	if not BankListOptions[GetRealmName()][1]["guild_checked"] then
		BankListOptions[GetRealmName()][1]["player_type"] = this:GetID();
		if TypesIndex == 7 then
			UIDropDownMenu_DisableDropDown(BankGUIPlayerDropDown);
		else
			UIDropDownMenu_EnableDropDown(BankGUIPlayerDropDown);
		end
		if TypesIndex == 4 or TypesIndex == 5 or TypesIndex == 7 then
			BankGUIDeleteButton:SetText(BANKGUI_DELETE);
			BankGUIDeleteButton:Enable();
		else
			BankGUIDeleteButton:SetText(BANKGUI_DELETES);
			if PlayerIndex == 1 then BankGUIDeleteButton:Disable()
			else BankGUIDeleteButton:Enable() end
		end
		if TypesIndex == 6 or TypesIndex == 8 then
			BankGUIDeleteButton:Disable();
		end
	end
	BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
end


function BankGUIPlayerDropDown_Initialize()
  local i, bank_gold, bank_gold_tot = 0, 0.0, 0.0;
  local playermenuinfo = UIDropDownMenu_CreateInfo();
  if not BankListOptions then return end
  if not BankListOptions[GetRealmName()] then return end
  if not BankListOptions[GetRealmName()][1] then return end
  if BankListOptions[GetRealmName()][1]["guild_checked"] then
	if not BankListGuildInfo then return end
	if not BankListGuildInfo[GetRealmName()] then return end
	if not BankListOptions[GetRealmName()][1]["guild_name"] then
		BankListOptions[GetRealmName()][1]["guild_name"] = BankGUI:GetGuild(1);
	end
	for i = 1, BankGUI:GetNumGuilds() do
		if BankListGuildInfo[GetRealmName()][i]["money"] then
			if BankListGuildInfo[GetRealmName()][i]["name"] == BankGUI:GetGuild(PlayerIndex) then bank_gold_tot = (BankListGuildInfo[GetRealmName()][i]["money"]/10000) end
			bank_gold = BankListGuildInfo[GetRealmName()][i]["money"]/10000;
		else bank_gold = 0.0 end
		playermenuinfo.text = format("%s (%.2f G)", BankListGuildInfo[GetRealmName()][i]["name"], bank_gold);
		playermenuinfo.value = BankListGuildInfo[GetRealmName()][i]["name"];
		playermenuinfo.func = BankGUIPlayerDropDown_Select;
		playermenuinfo.owner = this:GetParent();
		playermenuinfo.checked = nil;
		UIDropDownMenu_AddButton(playermenuinfo);
 	end
	BankGUITotalGold:SetText( format(BANKGUI_GOLD_GUILD, bank_gold_tot) );
  else
	if not BankListPlayers then return end
	if not BankListPlayers[GetRealmName()] then return end
	playermenuinfo.text = "All Characters";
	playermenuinfo.value = "All Characters";
	playermenuinfo.func = BankGUIPlayerDropDown_Select;
	playermenuinfo.owner = this:GetParent();
	playermenuinfo.checked = nil;
	UIDropDownMenu_AddButton(playermenuinfo);
	for i = 1, table.getn(BankListPlayers[GetRealmName()]) do
	  if (BankListOptions[GetRealmName()][1][BANKGUI_FACTION]) or (BankListPlayers[GetRealmName()][i]["faction"] and BankListPlayers[GetRealmName()][i]["faction"] == bank_playerfaction) then
		if BankListPlayers[GetRealmName()][i]["money"] then
			bank_gold_tot = bank_gold_tot + (BankListPlayers[GetRealmName()][i]["money"]/10000);
			bank_gold = BankListPlayers[GetRealmName()][i]["money"]/10000;
		else bank_gold = 0.0 end
		playermenuinfo.text = format("%s (%.2f G)", BankListPlayers[GetRealmName()][i]["name"], bank_gold);
		playermenuinfo.value = BankListPlayers[GetRealmName()][i]["name"];
		playermenuinfo.func = BankGUIPlayerDropDown_Select;
		playermenuinfo.owner = this:GetParent();
		playermenuinfo.checked = nil;
		UIDropDownMenu_AddButton(playermenuinfo);
	  end
 	end
	BankGUITotalGold:SetText( format(BANKGUI_GOLD_BANK, bank_gold_tot) );
  end
end


function BankGUIPlayerDropDown_Select()
  UIDropDownMenu_SetSelectedID(this.owner, this:GetID());
  PlayerIndex = this:GetID();
  if BankListOptions[GetRealmName()][1]["guild_checked"] then
	SelectedID = BankGUI:GetGuild(PlayerIndex);
	BankListOptions[GetRealmName()][1]["guild_name"] = SelectedID;
	UIDropDownMenu_SetText(BankGUIPlayerDropDown, SelectedID);
	if SelectedID == "" then
		BankGUITotalGold:SetText( format(BANKGUI_GOLD_GUILD,0.0) );
		BankGUIDeleteButton:Disable();
	else
		BankGUITotalGold:SetText( format(BANKGUI_GOLD_GUILD,(BankListGuildInfo[GetRealmName()][PlayerIndex]["money"]/10000)) );
		BankGUIDeleteButton:Enable();
	end
	BankGUIDeleteButton:SetText(BANKGUI_DELETES);
  else
	if PlayerIndex == 1 then
		SelectedID = nil;
		BankListOptions[GetRealmName()][1]["player_name"] = nil;
		if TypesIndex == 4 or TypesIndex == 5 then BankGUIDeleteButton:Enable()
		else BankGUIDeleteButton:Disable() end
	else
		local i, num_matchesfound = 0, 0;
		for i = 1, table.getn(BankListPlayers[GetRealmName()]) do
			if (BankListOptions[GetRealmName()][1][BANKGUI_FACTION]) or (BankListPlayers[GetRealmName()][i]["faction"] and BankListPlayers[GetRealmName()][i]["faction"] == bank_playerfaction) then
				num_matchesfound = num_matchesfound + 1;
			end
			if num_matchesfound == PlayerIndex-1 then
				SelectedID = BankListPlayers[GetRealmName()][i]["name"];
				break;
			end
		end
		UIDropDownMenu_SetText(BankGUIPlayerDropDown, SelectedID);
		BankGUIDeleteButton:Enable();
		BankListOptions[GetRealmName()][1]["player_name"] = SelectedID;
	end
	if TypesIndex == 4 or TypesIndex == 5 then BankGUIDeleteButton:SetText(BANKGUI_DELETE)
	else BankGUIDeleteButton:SetText(BANKGUI_DELETES)
	end
	if TypesIndex == 6 or TypesIndex == 7 or TypesIndex == 8 then
		BankGUIDeleteButton:Disable();
	end
  end
  BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
end


function BankGUILinkDropDown_Initialize(self, level)
  local linkmenuinfo = UIDropDownMenu_CreateInfo();
  local i, friendsonline, name, lvl, class, area, connected = 0, false;
  if level == nil then level = 1 end
  if level == 1 then
	for i = 1, table.getn(linktochattypes) do
		linkmenuinfo.text = linktochattypes[i][1];
		linkmenuinfo.value = { ["Level1_Key"] = linktochattypes[i][1] };
		linkmenuinfo.func = BankGUILinkDropDown_Select;
		linkmenuinfo.owner = this:GetParent();
		linkmenuinfo.checked = nil;
		linkmenuinfo.isTitle = linktochattypes[i][3];
		linkmenuinfo.hasArrow = linktochattypes[i][2];
		if linkmenuinfo.hasArrow then linkmenuinfo.nested = 1 else linkmenuinfo.nested = 0 end
		linkmenuinfo.notCheckable = true;
		linkmenuinfo.disabled = nil;
		if linktochattypes[i][1] == "FRIEND" then
			for i = 1, GetNumFriends() do
				name, lvl, class, area, connected = GetFriendInfo(i);
				if name and connected then friendsonline = true end
			end
			if not friendsonline then linkmenuinfo.disabled = 1 end
		elseif linktochattypes[i][1] == "WHISPERS" then
			if table.getn(Wisp_Data) == 0 then linkmenuinfo.disabled = 1 end
		elseif linktochattypes[i][1] == "RAID" then
			if GetNumRaidMembers() == 0 then linkmenuinfo.disabled = 1 end
		elseif linktochattypes[i][1] == "PARTY" then
			if GetNumPartyMembers() == 0 then linkmenuinfo.disabled = 1 end
		elseif linktochattypes[i][1] == "GUILD" then
			if not GetGuildInfo("player") then linkmenuinfo.disabled = 1 end
		end
		UIDropDownMenu_AddButton(linkmenuinfo, level);
 	end
  elseif level == 2 then
	ShowFriends();
	local friend, name, lvl, class, area, connected;
	local Level1_Key = UIDROPDOWNMENU_MENU_VALUE["Level1_Key"];
	Link_Data[Level1_Key] = {};
	if Level1_Key == "FRIEND" then
		for i = 1, GetNumFriends() do
			name, lvl, class, area, connected = GetFriendInfo(i);
			if name and connected then
				friend = { ["name"] = name; };
				table.insert(Link_Data[Level1_Key], friend);
			end
		end
	else
		for i = 1, table.getn(Wisp_Data) do
			friend = Wisp_Data[i];
			table.insert(Link_Data[Level1_Key], friend);
		end
	end
	for i = 1, table.getn(Link_Data[Level1_Key]) do
		linkmenuinfo.text = Link_Data[Level1_Key][i]["name"];
		linkmenuinfo.value = { ["Level1_Key"] = Level1_Key; ["Sublevel_Key"] = Link_Data[Level1_Key][i]["name"]; };
		linkmenuinfo.func = BankGUIFriend_Select;
		linkmenuinfo.owner = this:GetParent();
		linkmenuinfo.checked = nil;
		linkmenuinfo.hasArrow = false;
		linkmenuinfo.notCheckable = true;
		linkmenuinfo.isTitle = false;
		linkmenuinfo.disabled = false;
		UIDropDownMenu_AddButton(linkmenuinfo, level);
 	end
  end
end


function BankGUILinkDropDown_Select()
	local bankinfo = BankGUI:GetItemByIndex(ClickedIndex);
	if linktochattypes[this:GetID()] and linktochattypes[this:GetID()][2] == nil then
		if bankinfo["type"] == BANKGUI_TYPE_TRADELINK then
			SendChatMessage( format(BANKGUI_TITLE ..": %s %s (%s)", bankinfo["name"], bankinfo["item"], bankinfo["amount"]), linktochattypes[this:GetID()][1], nil)
		else
			SendChatMessage(bankinfo["item"], linktochattypes[this:GetID()][1], nil)
		end
	end
	CloseDropDownMenus();
end


function BankGUIFriend_Select()
	local bankinfo = BankGUI:GetItemByIndex(ClickedIndex);
	if bankinfo["type"] == BANKGUI_TYPE_TRADELINK then
		SendChatMessage( format(BANKGUI_TITLE ..": %s %s (%s)", bankinfo["name"], bankinfo["item"], bankinfo["amount"]), "WHISPER", nil, UIDropDownMenuButton_GetName(this));
	else
		SendChatMessage(bankinfo["item"], "WHISPER", nil, UIDropDownMenuButton_GetName(this));
	end
	CloseDropDownMenus();
end


function BankGUIDetailsIcon_OnClick(button)
	if (button == "LeftButton") then
		local bankinfo = BankGUI:GetItemByIndex(SelectedIndex);
		if IsShiftKeyDown() then
			if ChatFrameEditBox:IsVisible() then ChatFrameEditBox:Insert(bankinfo["item"])
			else bank_chat:AddMessage(bankinfo["item"])
			end
		elseif (IsControlKeyDown()) then DressUpItemLink(bankinfo["item"])
		else bank_chat:AddMessage(bankinfo["item"])
		end
	end
end


function BankGUIDetailsIcon_OnEnter()
	local bankinfo = BankGUI:GetItemByIndex(SelectedIndex);
	GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT");
	GameTooltip:ClearLines();
	GameTooltip:SetHyperlink(bankinfo["item"]);
end


local bank_farm_player = UnitName("player");

function BankGUIFarmFrame_OnShow()
	UIDropDownMenu_SetText(BankGUIFarmPlayerDropDown, bank_farm_player);
	BankGUIFarmItemBox:SetFocus();
end


function BankGUIFarmPlayerDropDown_Initialize()
	local i, farmmenuinfo = 0, UIDropDownMenu_CreateInfo();
	if not BankListPlayers then return end
	if not BankListPlayers[GetRealmName()] then return end
	for i = 1, table.getn(BankListPlayers[GetRealmName()]) do
		farmmenuinfo.text = BankListPlayers[GetRealmName()][i]["name"];
		farmmenuinfo.value = BankListPlayers[GetRealmName()][i]["name"];
		farmmenuinfo.func = BankGUIFarmPlayerDropDown_Select;
		farmmenuinfo.owner = this:GetParent();
		farmmenuinfo.checked = nil;
		if BankListPlayers[GetRealmName()][i]["name"] == bank_farm_player then
			k = i;
			farmmenuinfo.checked = 1;
		end
		UIDropDownMenu_AddButton(farmmenuinfo);
 	end
end


function BankGUIFarmPlayerDropDown_Select()
	UIDropDownMenu_SetSelectedID(this.owner, this:GetID());
	bank_farm_player = this.value;
end


local bank_farm_item;

function BankGUIFarmItemBox_OnText()
	CloseDropDownMenus();
	bank_farm_item = this:GetText();
	if string.len(bank_farm_item) == 0 then return end
	BankGUIFarmItemDropDown.initialize = BankGUIFarmItemDropDown_Initialize;
	BankGUIFarmItemDropDown.displayMode = "MENU";
	ToggleDropDownMenu(1, nil, BankGUIFarmItemDropDown, "BankGUIFarmItemBox", 0, 0);
end


function BankGUIFarmItemDropDown_Initialize()
	local i, k, itemName = 0, 0, "";
	local farmitemmenuinfo = UIDropDownMenu_CreateInfo();
	for i = 1, table.getn(BankListItems[GetRealmName()]) do
		itemName = GetItemInfo(BankListItems[GetRealmName()][i]["item"]);
		if itemName and k < 25 and BankListItems[GetRealmName()][i]["bank"] and string.find(string.lower(itemName), bank_farm_item) ~= nil then
			farmitemmenuinfo.text = BankListItems[GetRealmName()][i]["item"];
			farmitemmenuinfo.value = i;
			farmitemmenuinfo.func = BankGUIFarmItemDropDown_Select;
			farmitemmenuinfo.owner = this:GetParent();
			farmitemmenuinfo.checked = nil;
			farmitemmenuinfo.hasArrow = false;
			farmitemmenuinfo.notCheckable = false;
			farmitemmenuinfo.isTitle = false;
			farmitemmenuinfo.disabled = nil;
			UIDropDownMenu_AddButton(farmitemmenuinfo);
			k = k + 1;
		end
 	end
end


function BankGUIFarmItemDropDown_Select()
	BankGUIFarmItemBox:SetText("");
	BankGUIFarmItemBox:Insert(BankListItems[GetRealmName()][this.value]["item"]);
end


function BankGUIFarmAddItem_OnClick()
	local farminfo, itemname, itemlink, itemstack = {};
	itemname, itemlink = GetItemInfo(BankGUIFarmItemBox:GetText());
	farminfo["name"] = bank_farm_player;
	if itemlink then farminfo["item"] = itemlink
	else farminfo["item"] = BankGUIFarmItemBox:GetText() end
	farminfo["amount"] = BankGUIFarmAmountBox:GetNumber();
	farminfo["date"] = date("%d/%m/%y");
	if farminfo["item"] ~= "" and farminfo["amount"] > 0 then
		PlaySound("igMainMenuOptionCheckBoxOn");
		table.insert(BankListFarms[GetRealmName()], farminfo);
		itemname, _, _, _, _, _, _, itemstack = GetItemInfo(farminfo["item"]);
		if not itemname then
			itemname = BankGUIFarmItemBox:GetText();
			itemstack = 20;
		end
		if itemname and itemstack > 1 then
			if not BankListStock[itemname] then BankListStock[itemname] = {} end
			if not BankListStock[itemname]["farm"] then BankListStock[itemname]["farm"] = {} end
			if not BankListStock[itemname]["farm"]["amount"] then BankListStock[itemname]["farm"]["amount"] = 0 end
			if not BankListStock[itemname]["farm"]["date"] then BankListStock[itemname]["farm"]["date"] = date("%d/%m/%y") end
			BankListStock[itemname]["farm"]["amount"] = BankListStock[itemname]["farm"]["amount"] + farminfo["amount"];
		end
		BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText());
	end
end


function BankGUI:UpdateUI()
	local numItems = BankGUI:GetNumItems();
	local blIndex, blOffset, blButtxt, blButton, blColor, bankinfo, disp, i, itemStackCount, itemTexture;
	local selectedBankGUI = BankGUI:GetSelectedBankGUI();

	FauxScrollFrame_Update(getglobal("BankGUIScrollFrame"), numItems, BANKGUIS_TO_DISPLAY, FRIENDS_FRAME_BANKGUI_HEIGHT, nil, nil, nil, nil, nil, nil, true);
	blOffset = FauxScrollFrame_GetOffset(getglobal("BankGUIScrollFrame"));

	if numItems == 0 then
		for i = 1, BANKGUIS_TO_DISPLAY, 1 do
			blButton = getglobal("BankGUIButton" .. i);
			blButton:Hide();
		end
		return;
	end

	if (numItems > 0) then
		if (selectedBankGUI == 0 or selectedBankGUI > numItems) then
			BankGUI:SetSelectedBankGUI(1);
			selectedBankGUI = 1;
		end
	end

	for i = 1, BANKGUIS_TO_DISPLAY, 1 do
		blIndex = i + blOffset;
		disp = "";
		blColor = "|cffffffff";
		bankinfo = BankGUI:GetItemByIndex(blIndex);
		blButtxt = getglobal("BankGUIButton" .. i .. "TextLabel");
		blButton = getglobal("BankGUIButton" .. i);
		if bankinfo then
			if bankinfo["type"] == BANKGUI_TYPE_TRADELINK then
				disp = format("%s%s|r %s %s(%s)|r", bankinfo["color"], bankinfo["name"], bankinfo["item"], bankinfo["color"], bankinfo["amount"]);
			elseif bankinfo["type"] == BANKGUI_TYPE_FARMS then
				if not bankinfo["date"] then bankinfo["date"] = "" end
				disp = format("%s %s(%i)|r |cff808080%s|r", bankinfo["item"], blColor, bankinfo["amount"], bankinfo["date"]);
			elseif bankinfo["type"] == BANKGUI_TYPE_GLYPHS then
				if bankinfo["glyp"] == 1 then blColor = "|cff00ffff"
				else blColor = "|cff00a0a0"
				end
				disp = format("%s%s spec %i, slot %i|r", bankinfo["item"], blColor, bankinfo["spec"], bankinfo["slot"]);
			elseif bankinfo["type"] == BANKGUI_TYPE_FRIENDS then
				disp = bankinfo["name"];
				if bankinfo["level"] ~= nil then
					disp = disp ..format(" |cffffffff(%i, %s)|r", bankinfo["level"], bankinfo["class"]);
				end
				if bankinfo["note"] ~= nil then
					disp = disp ..format(" |cffffff80%s|r", bankinfo["note"]);
				end
			elseif bankinfo["type"] == BANKGUI_TYPE_CURRENCY then
				if bankinfo["amount"] == nil then bankinfo["amount"] = 0 end
				if bankinfo["item"] == nil then bankinfo["item"] = "" end
				disp = format("%s%s|r (%i) |cffffff80%s|r", blColor, bankinfo["item"], bankinfo["amount"], bankinfo["name"]);
			else
				_, _, _, _, _, _, _, itemStackCount = GetItemInfo(bankinfo["item"]);
				if itemStackCount == nil then itemStackCount = 1 end
				disp = bankinfo["item"];
				if itemStackCount ~= 1 then disp = disp ..format(" (%i)", bankinfo["amount"]) end
			end
		end
		blButtxt:SetText(disp);
		blButton:SetID(blIndex);

		-- Update the highlight
		if (blIndex == selectedBankGUI) then
			blButton:LockHighlight();
		else
			blButton:UnlockHighlight();
		end

		if (blIndex > numItems) then
			blButton:Hide();
		else
			blButton:Show();
		end
	end
end


local itema, itemb, itemalvl, itemblvl, itemamin, itembmin, amounta, amountb, sortvala, sortvalb, sortstringa, sortstringb;

local function BankGUI_GetSortValue1(sortvalue, sortvalueA, sortvalueB, a, b)
	if sortvalue == BANKGUI_SORTNAME1 then
		if itema == itemb then
			return sortvalueA, sortvalueB
		elseif itema < itemb then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	elseif sortvalue == BANKGUI_SORTNAME2 then
		if a["name"] == b["name"] then
			return sortvalueA, sortvalueB
		elseif a["name"] < b["name"] then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	elseif sortvalue == BANKGUI_SORTNAME3 then
		if amounta == amountb then
			return sortvalueA, sortvalueB
		elseif amounta > amountb then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	elseif sortvalue == BANKGUI_SORTNAME4 then
		if (itemalvl == itemblvl) and (itemamin == itembmin) then
			return sortvalueA, sortvalueB
		elseif (itemalvl > itemblvl) or (itemalvl == itemblvl and itemamin > itembmin) then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	end
	return sortvalueA, sortvalueB;
end


local function BankGUI_GetSortValue2(sortvalue, sortvalueA, sortvalueB, a, b)
	if sortvalue == BANKGUI_SORTNAME1 then
		if itema == itemb then
			return sortvalueA, sortvalueB
		elseif itema < itemb then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	elseif sortvalue == BANKGUI_SORTNAME2 then
		if a["name"] == b["name"] then
			return sortvalueA, sortvalueB
		elseif a["name"] < b["name"] then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	elseif sortvalue == BANKGUI_SORTNAME3 then
		if amounta == amountb then
			return sortvalueA, sortvalueB
		elseif amounta > amountb then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	elseif sortvalue == BANKGUI_SORTNAME4 then
		if (itemalvl == itemblvl) and (itemamin == itembmin) then
			return sortvalueA, sortvalueB
		elseif (itemalvl > itemblvl) or (itemalvl == itemblvl and itemamin > itembmin) then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	end
	return sortvalueA, sortvalueB;
end


local function BankGUI_GetSortValue3(sortvalue, sortvalueA, sortvalueB, a, b)
	if sortvalue == BANKGUI_SORTNAME1 then
		if itema == itemb then
			return sortvalueA, sortvalueB
		elseif itema < itemb then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	elseif sortvalue == BANKGUI_SORTNAME2 then
		if a["name"] == b["name"] then
			return sortvalueA, sortvalueB
		elseif a["name"] < b["name"] then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	elseif sortvalue == BANKGUI_SORTNAME3 then
		if amounta == amountb then
			return sortvalueA, sortvalueB
		elseif amounta > amountb then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	elseif sortvalue == BANKGUI_SORTNAME4 then
		if (itemalvl == itemblvl) and (itemamin == itembmin) then
			return sortvalueA, sortvalueB
		elseif (itemalvl > itemblvl) or (itemalvl == itemblvl and itemamin > itembmin) then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	end
	return sortvalueA, sortvalueB;
end


local function BankGUI_SortTotals()
	if BankListOptions[GetRealmName()][1][BANKGUI_TOTALS] == true then
		for subarray in pairs(BankTotals) do
			for subsubarray in pairs(BankTotals[subarray]) do
				bankinfo = {};
				for subsubsub in pairs(BankTotals[subarray][subsubarray]) do
					bankinfo[subsubsub] = BankTotals[subarray][subsubarray][subsubsub];
				end
				table.insert(BankFilter, bankinfo);
			end
		end
	end
	if  BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE1]["name"] == BANKGUI_SORTNAME5
	and BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE2]["name"] == BANKGUI_SORTNAME5
	and BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE3]["name"] == BANKGUI_SORTNAME5 then
		return
	end
	table.sort(BankFilter, function(a, b)
		local i;	
		itema, _, _, itemalvl, itemamin = GetItemInfo(a["item"]);
		itemb, _, _, itemblvl, itembmin = GetItemInfo(b["item"]);
		if itema == nil then
			if a["type"] == BANKGUI_TYPE_TRADELINK then
				i = string.find(a["item"], "|Htrade");
				if i == nil then itema = a["item"]
				else
					i = string.find(a["item"], "|h", i);
					i = i + 3;
					itema = string.sub(a["item"], i, string.len(a["item"]) - 5);
				end
			else
				itema = a["item"]
			end
		end
		if itemb == nil then
			if b["type"] == BANKGUI_TYPE_TRADELINK then
				i = string.find(b["item"], "|Htrade");
				if i == nil then itemb = b["item"]
				else
					i = string.find(b["item"], "|h", i);
					i = i + 3;
					itemb = string.sub(b["item"], i, string.len(b["item"]) - 5);
				end
			else
				itemb = b["item"]
			end
		end
		if itemalvl == nil then itemalvl = 0 end
		if itemblvl == nil then itemblvl = 0 end
		if itemamin == nil then itemamin = 0 end
		if itembmin == nil then itembmin = 0 end
		amounta = tonumber(a["amount"]);
		amountb = tonumber(b["amount"]);
		if amounta == nil then amounta = 0 end
		if amountb == nil then amountb = 0 end
		sortvala = 0; sortvalb = 0;
		if BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE1]["name"] ~= BANKGUI_SORTNAME5 then
			sortvala, sortvalb = BankGUI_GetSortValue1(BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE1]["name"], sortvala, sortvalb, a, b)
		end
		if BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE2]["name"] ~= BANKGUI_SORTNAME5 then
			sortvala, sortvalb = BankGUI_GetSortValue2(BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE2]["name"], sortvala, sortvalb, a, b)
		end
		if BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE3]["name"] ~= BANKGUI_SORTNAME5 then
			sortvala, sortvalb = BankGUI_GetSortValue3(BankListOptions[GetRealmName()][1][BANKGUI_SORTTITLE3]["name"], sortvala, sortvalb, a, b)
		end
		return sortvala > sortvalb;
	end );
end


function BankGUI:UpdateFilter(filter)
  local i, k, matchfound, searchstring, filterstring, bankinfo, name, online, itemName, itemRarity, itemType, itemSubType, itemEquipLoc, subarray, subsubarray, subsubsub = 0, 0, false;
  if filter == nil then filterstring = "" else filterstring = string.lower(filter) end
  while table.getn(BankFilter) > 0 do
	table.remove(BankFilter, table.getn(BankFilter));
  end
  BankFilter = {};
  BankTotals = {};
  if BankListOptions[GetRealmName()][1]["guild_checked"] then
	if not BankListGuildItems[GetRealmName()] then BankListGuildItems[GetRealmName()] = {} end
	if not BankListGuildItems[GetRealmName()][SelectedID] then BankListGuildItems[GetRealmName()][SelectedID] = {} end
	for i = 1, table.getn(BankListGuildItems[GetRealmName()][SelectedID]) do
		matchfound = false;
		searchstring = "";
		bankinfo = {};
		bankinfo["name"] = SelectedID;
		for subarray in pairs(BankListGuildItems[GetRealmName()][ bankinfo["name"] ][i]) do
			bankinfo[subarray] = BankListGuildItems[GetRealmName()][ bankinfo["name"] ][i][subarray];
		end
		bankinfo["bank"] = true;
		itemName, _, itemRarity, _, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(bankinfo["item"]);
		if itemName == nil then itemName = bankinfo["item"] end
		if filterstring == "" then matchfound = true
		else
			searchstring = bankinfo["item"];
			if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
			if itemType then searchstring = searchstring .." " ..itemType end
			if itemSubType then searchstring = searchstring .." " ..itemSubType end
			if itemRarity then searchstring = searchstring .." " ..itemqualitystrings[itemRarity+1] end
			if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
		end
		if matchfound then
			if not BankListOptions[GetRealmName()][1][BANKGUI_TOTALS] then
				table.insert(BankFilter, bankinfo);
			end
			if not BankTotals[ bankinfo["name"] ] then BankTotals[ bankinfo["name"] ] = { } end
			if not BankTotals[ bankinfo["name"] ][itemName] then
				BankTotals[ bankinfo["name"] ][itemName] = { };
				for subarray in pairs(bankinfo) do
					BankTotals[ bankinfo["name"] ][itemName][subarray] = bankinfo[subarray];
				end
			else
				BankTotals[ bankinfo["name"] ][itemName]["amount"] = BankTotals[ bankinfo["name"] ][itemName]["amount"] + bankinfo["amount"]
			end
		end
	end
	BankGUI_SortTotals();
  else
	for i = 1, table.getn(BankListItems[GetRealmName()]) do
	  if (BankListOptions[GetRealmName()][1][BANKGUI_FACTION]) or (BankFactionColors[ BankListItems[GetRealmName()][i]["name"] ] and BankFactionColors[ BankListItems[GetRealmName()][i]["name"] ]["faction"] and BankFactionColors[ BankListItems[GetRealmName()][i]["name"] ]["faction"] == bank_playerfaction) then
		matchfound = false;
		searchstring = "";
		bankinfo = {};
		for subarray in pairs(BankListItems[GetRealmName()][i]) do
			bankinfo[subarray] = BankListItems[GetRealmName()][i][subarray];
		end
		itemName, _, itemRarity, _, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(bankinfo["item"]);
		if itemName == nil then itemName = bankinfo["item"] end
		if ( TypesIndex == 1 or (TypesIndex == 2 and bankinfo["bank"] and BankGUI:IsValidType(bankinfo)) or (TypesIndex == 3 and not bankinfo["bank"] and BankGUI:IsValidType(bankinfo)) or (TypesIndex == 4 and bankinfo["type"] == BANKGUI_TYPE_TRADELINK) ) then
			if SelectedID == nil then
				if filterstring == "" then matchfound = true
				else
					searchstring = bankinfo["item"];
					if bankinfo["name"] ~= nil then searchstring = searchstring ..bankinfo["name"] end
					if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
					if itemType then searchstring = searchstring .." " ..itemType end
					if itemSubType then searchstring = searchstring .." " ..itemSubType end
					if itemRarity then searchstring = searchstring .." " ..itemqualitystrings[itemRarity+1] end
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			elseif filterstring == "" then
				if bankinfo["name"] == SelectedID then matchfound = true end
			else
				if string.find(bankinfo["name"], SelectedID) ~= nil then
					searchstring = bankinfo["item"];
					if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
					if itemType then searchstring = searchstring .." " ..itemType end
					if itemSubType then searchstring = searchstring .." " ..itemSubType end
					if itemRarity then searchstring = searchstring .." " ..itemqualitystrings[itemRarity+1] end
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			end
			if matchfound then
				if not BankListOptions[GetRealmName()][1][BANKGUI_TOTALS] then
					table.insert(BankFilter, bankinfo);
				end
				if not BankTotals[ bankinfo["name"] ] then BankTotals[ bankinfo["name"] ] = { } end
				if not BankTotals[ bankinfo["name"] ][itemName] then
					BankTotals[ bankinfo["name"] ][itemName] = { };
					for subarray in pairs(bankinfo) do
						BankTotals[ bankinfo["name"] ][itemName][subarray] = bankinfo[subarray];
					end
				else
					BankTotals[ bankinfo["name"] ][itemName]["amount"] = BankTotals[ bankinfo["name"] ][itemName]["amount"] + bankinfo["amount"]
				end
			end
		end
	  end
	end
	BankGUI_SortTotals();
	professioncolors["GUILDON"] = bank_guildcolor_on;
	professioncolors["GUILDOFF"] = bank_guildcolor_of;
	for i = 1, table.getn(BankFilter) do
		BankFilter[i]["color"] = professioncolors["NONE"];
		for k = 1, table.getn(BankListPlayers[GetRealmName()]) do
			if BankFilter[i]["name"] == BankListPlayers[GetRealmName()][k]["name"] then BankFilter[i]["color"] = professioncolors["ALT"] end
		end
		if BankFilter[i]["color"] == professioncolors["NONE"] then
			for k = 1, GetNumGuildMembers(true) do
				name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(k);
				if BankFilter[i]["name"] == name then
					if online then
						BankFilter[i]["color"] = professioncolors["GUILDON"];
					else
						BankFilter[i]["color"] = professioncolors["GUILDOFF"];
					end
				end
			end
		end
		if BankFilter[i]["color"] == professioncolors["NONE"] then
			for k = 1, GetNumFriends() do
				name, _, _, _, online = GetFriendInfo(k);
				if BankFilter[i]["name"] == name and online then
					BankFilter[i]["color"] = professioncolors["FRIENDON"];
				end
			end
		end
	end
	if ( TypesIndex == 1 or TypesIndex == 5 ) then
		for i = 1, table.getn(BankListFarms[GetRealmName()]) do
		  if (BankListOptions[GetRealmName()][1][BANKGUI_FACTION]) or (BankFactionColors[ BankListFarms[GetRealmName()][i]["name"] ] and BankFactionColors[ BankListFarms[GetRealmName()][i]["name"] ]["faction"] and BankFactionColors[ BankListFarms[GetRealmName()][i]["name"] ]["faction"] == bank_playerfaction) then
			bankinfo = {};
			matchfound = false;
			searchstring = "";
			bankinfo["name"] = BankListFarms[GetRealmName()][i]["name"];
			bankinfo["item"] = BankListFarms[GetRealmName()][i]["item"];
			bankinfo["amount"] = BankListFarms[GetRealmName()][i]["amount"];
			bankinfo["date"] = BankListFarms[GetRealmName()][i]["date"];
			bankinfo["type"] = BANKGUI_TYPE_FARMS;
			if SelectedID == nil then
				if filterstring == "" then matchfound = true
				else
					searchstring = bankinfo["name"] ..bankinfo["item"];
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			elseif filterstring == "" then
				if bankinfo["name"] == SelectedID then matchfound = true end
			else
				if bankinfo["name"] == SelectedID then
					searchstring = bankinfo["item"];
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			end
			if matchfound == true then table.insert(BankFilter, bankinfo) end
		  end
		end
	end
	if ( TypesIndex == 1 or TypesIndex == 6 ) then
		for i = 1, table.getn(BankListGlyphs[GetRealmName()]) do
		  if (BankListOptions[GetRealmName()][1][BANKGUI_FACTION]) or (BankFactionColors[ BankListGlyphs[GetRealmName()][i]["name"] ] and BankFactionColors[ BankListGlyphs[GetRealmName()][i]["name"] ]["faction"] and BankFactionColors[ BankListGlyphs[GetRealmName()][i]["name"] ]["faction"] == bank_playerfaction) then
			bankinfo = {};
			matchfound = false;
			searchstring = "";
			bankinfo["name"] = BankListGlyphs[GetRealmName()][i]["name"];
			bankinfo["spec"] = BankListGlyphs[GetRealmName()][i]["spec"];
			bankinfo["item"] = BankListGlyphs[GetRealmName()][i]["item"];
			if not BankListGlyphs[GetRealmName()][i]["glyp"] then BankListGlyphs[GetRealmName()][i]["glyp"] = 0 end
			bankinfo["glyp"] = BankListGlyphs[GetRealmName()][i]["glyp"];
			bankinfo["slot"] = BankListGlyphs[GetRealmName()][i]["slot"];
			bankinfo["type"] = BANKGUI_TYPE_GLYPHS;
			if SelectedID == nil then
				if filterstring == "" then matchfound = true
				else
					searchstring = bankinfo["name"] ..bankinfo["item"];
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			elseif filterstring == "" then
				if bankinfo["name"] == SelectedID then matchfound = true end
			else
				if bankinfo["name"] == SelectedID then
					searchstring = bankinfo["item"];
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			end
			if matchfound == true then table.insert(BankFilter, bankinfo) end
		  end
		end
	end
	if ( TypesIndex == 1 or TypesIndex == 7 ) then
		for i = 1, table.getn(BankListFriends[GetRealmName()]) do
		  if (BankListOptions[GetRealmName()][1][BANKGUI_FACTION]) or (BankListFriends[GetRealmName()][i]["faction"] and BankListFriends[GetRealmName()][i]["faction"] == bank_playerfaction) then
			bankinfo = {};
			matchfound = false;
			searchstring = "";
			bankinfo["name"] = BankListFriends[GetRealmName()][i]["name"];
			bankinfo["item"] = BankListFriends[GetRealmName()][i]["name"];
			bankinfo["level"] = BankListFriends[GetRealmName()][i]["level"];
			bankinfo["class"] = BankListFriends[GetRealmName()][i]["class"];
			bankinfo["note"] = BankListFriends[GetRealmName()][i]["note"];
			bankinfo["faction"] = BankListFriends[GetRealmName()][i]["faction"];
			bankinfo["type"] = BANKGUI_TYPE_FRIENDS;
			if filterstring == "" then matchfound = true
			else
				searchstring = bankinfo["name"];
				if bankinfo["class"] ~= nil then searchstring = searchstring ..bankinfo["class"] end
				if bankinfo["note"] ~= nil then searchstring = searchstring ..bankinfo["note"] end
				if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
			end
			if matchfound == true then table.insert(BankFilter, bankinfo) end
		  end
		end
	end
	if ( TypesIndex == 1 or TypesIndex == 8 ) then
		for i = 1, table.getn(BankListCurrency[GetRealmName()]) do
		  if (BankListOptions[GetRealmName()][1][BANKGUI_FACTION]) or (BankFactionColors[ BankListCurrency[GetRealmName()][i]["name"] ] and BankFactionColors[ BankListCurrency[GetRealmName()][i]["name"] ]["faction"] and BankFactionColors[ BankListCurrency[GetRealmName()][i]["name"] ]["faction"] == bank_playerfaction) then
			bankinfo = {};
			matchfound = false;
			searchstring = "";
			bankinfo["name"] = BankListCurrency[GetRealmName()][i]["name"];
			bankinfo["item"] = BankListCurrency[GetRealmName()][i]["item"];
			bankinfo["amount"] = BankListCurrency[GetRealmName()][i]["amount"];
			bankinfo["type"] = BANKGUI_TYPE_CURRENCY;
			if SelectedID == nil then
				if filterstring == "" then matchfound = true
				else
					searchstring = bankinfo["name"] ..bankinfo["item"];
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			elseif filterstring == "" then
				if bankinfo["name"] == SelectedID then matchfound = true end
			else
				if bankinfo["name"] == SelectedID then
					searchstring = bankinfo["item"];
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
			end
			if matchfound == true then table.insert(BankFilter, bankinfo) end
		  end
		end
	end
  end
  BankGUIDetailsFrame:Hide();
  BankGUI:UpdateUI();
end


function BankGUI_Update()
	BankGUI:UpdateUI();
end
